package com.jiuqi.util;

package .tmp;

import java.util.ArrayList;
import java.util.List;

public class SparseList {
  private int size = 0;
  
  private int slotLength;
  
  private List slotsList;
  
  public SparseList(int estimateCapacity) {
    if (estimateCapacity < 10000) {
      this.slotLength = 128;
      this.slotsList = new ArrayList(estimateCapacity / 128 + 1);
    } else if (estimateCapacity <= 1000000) {
      this.slotLength = 1024;
      this.slotsList = new ArrayList(estimateCapacity / 1024 + 1);
    } else {
      this.slotLength = 8192;
      this.slotsList = new ArrayList(estimateCapacity / 8192 + 1);
    } 
  }
  
  public int size() {
    return this.size;
  }
  
  public void setSize(int value) {
    if (value >= 0) {
      this.size = value;
      if (this.size == 0)
        this.slotsList.clear(); 
    } 
  }
  
  public Object get(int index) {
    int slotIndex = index / this.slotLength;
    index %= this.slotLength;
    Object[] slot = (slotIndex >= this.slotsList.size()) ? null : this.slotsList.get(slotIndex);
    return (slot == null) ? null : slot[index];
  }
  
  public void set(int index, Object value) {
    if (index < 0 || index >= this.size)
      return; 
    int slotIndex = index / this.slotLength;
    index %= this.slotLength;
    for (; slotIndex <= this.slotsList.size(); this.slotsList.add(null));
    Object[] slot = this.slotsList.get(slotIndex);
    if (slot == null && value != null) {
      slot = new Object[this.slotLength];
      this.slotsList.set(slotIndex, slot);
    } 
    if (slot != null)
      slot[index] = value; 
  }
  
  public int add(Object value) {
    setSize(this.size + 1);
    set(this.size - 1, value);
    return this.size - 1;
  }
  
  public int getSlotLength() {
    return this.slotLength;
  }
  
  public void setSlotLengt(int value) {
    if (value > 1 && this.size == 0) {
      this.slotLength = value;
      this.slotsList.clear();
    } 
  }
}
